#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) since 2015 NTT DATA Corporation 
# Last update : 2015/09/15
#
from __future__ import unicode_literals

import argparse
import codecs
import docker
import logging.config
import os
import re
import sys
import traceback
import datetime
import logging
from logging.handlers import SysLogHandler

SCRIPT_NAME = os.path.splitext(os.path.basename(sys.argv[0]))[0]
DOCKER_API_VERSION = "1.18"
DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = "2375"
TMP_DIR = "/tmp"
TMP_FINGERPRINT = SCRIPT_NAME + "_1"
TMP_EXPIRES = 0 # disable

# 標準入出力をUTF-8で行う
sys.stdin = codecs.getreader("utf-8")(sys.stdin)
sys.stdout = codecs.getwriter("utf-8")(sys.stdout)

# カレントディレクトリをスクリプトのディレクトリへ移動
# loggingを初期化(logging.iniファイルがある場合)
os.chdir(os.path.dirname(os.path.abspath(sys.argv[0])))
if os.path.exists("logging.ini"): logging.config.fileConfig("logging.ini")
logger = logging.getLogger(SCRIPT_NAME)

class LocalException(Exception):
    pass

#---------------------------------------
# メイン
#---------------------------------------
def main():
    logger.info(SCRIPT_NAME)

    # 引数解析
    parser = argparse.ArgumentParser(
            formatter_class=argparse.RawTextHelpFormatter,
            description="Dockerのイベント情報を取得します。")

    parser.add_argument(
            "-i", dest="ip_addr", metavar="IP_ADDRESS",
            default=DEFAULT_HOST,
            help="Docker Remote APIの接続先IPアドレス。(default: '%s')" % DEFAULT_HOST)
    parser.add_argument(
            "-p", dest="port", metavar="DOCKER_PORT",
            default=DEFAULT_PORT,
            help="Docker Remote APIの接続先ポート。(default: '%s')" % DEFAULT_PORT)
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
            "-c", dest="cid", metavar="CONTAINER_ID", 
            help="コンテナのIDパターンを指定します。")
    group.add_argument(
            "-n", dest="name", metavar="CONTAINER_NAME",
            help="コンテナの名称パターンを指定します。")
    parser.add_argument(
            "-s", dest="syslog", metavar="SYSLOG_HOST:PORT", required=True,
            help="コンテナのステータスをsyslogとして送信します。")

    args = parser.parse_args()
    sa = args.syslog.split(":")
    send_contaner_status("http://%s:%s" % (args.ip_addr, args.port), sa[0],
                         int(sa[1]) if len(sa) > 1 else 514, args)
    return 0

#---------------------------------------
# コンテナのステータスを送信する
#---------------------------------------
def send_contaner_status(docker_url, syslog_ip, syslog_port, args):
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    syslog = SysLogHandler(address=(syslog_ip, syslog_port), facility=SysLogHandler.LOG_USER)
    logger.addHandler(syslog)
  
    dc = docker.Client(base_url=docker_url, version=DOCKER_API_VERSION)
    
    def send_contaner_status_to_syslog(cid):
      r = dc.inspect_container(cid)['State']
      s = 'running' if r['Running'] else ('paused' if r['Paused'] else ('restarting' if r['Restarting'] else 'stopped'))
      mt = '%s %s %s' % (datetime.datetime.today().strftime("%Y-%m-%d %H:%M:%S"), cid[0:12], s) 
      ms = '%s %s' % (cid[0:12], s)
      print mt
      
      logger.info(ms)
        
    containers = []
    if args.cid :
        pa = re.compile(('%s.*' % args.cid[:len(args.cid) - 1]) if args.cid.endswith('*') else args.cid)
        def find_container_by_id(c):
            if pa.match(c["Id"]):
                containers.append(c["Id"])
        map(find_container_by_id, dc.containers(all=True, quiet=True))
    elif args.name :
        pa = re.compile(('%s.*' % args.name[:len(args.name) - 1]) if args.name.endswith('*') else args.name)
        def find_container_by_name(c):
            # 名前に含まれる '/' を除外するため、マッチングする文字列の 1 文字目をスキップ
            matched_names = [n[1:] for n in c["Names"] if pa.match(n[1:])]
            if len(matched_names) > 0:
                containers.append(c["Id"])
        map(find_container_by_name, dc.containers(all=True))
    else :
        def all_container(c):
            containers.append(c["Id"])
        map(all_container,dc.containers(all=True))
    
    if len(containers) == 0:
        raise LocalException("指定されたコンテナは存在しません。")
    map(send_contaner_status_to_syslog, containers)
   

#===============================================================================
if __name__ == "__main__":
    try:
        sys.exit(main())

    # 予期されている例外はメッセージのみ出力
    except LocalException as e:
        print e.message
        sys.exit(1)

    # 予期しない例外はスタックトレースまで出力
    except Exception as e:
        print e.message
        traceback.print_exc()
        sys.exit(1)
